''' Launches klayout application from the command line to do dataprep stuff
'''
import argparse
import os
import subprocess

from lymask import batch_main
from lymask import __version__

parser = argparse.ArgumentParser(description="Command line mask dataprep")
parser.add_argument('infile', type=argparse.FileType('rb'),
                    help='the input gds file')
parser.add_argument('ymlfile', nargs='?', default=None,
                    help='YML file that describes the dataprep steps and parameters')
parser.add_argument('outfile', nargs='?', default=None,
                    help='The output file. Default is to tack "_proc" onto the end')
parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{__version__}')


def cm_main():
    ''' This calls klayout from command line using global variables '''
    args = parser.parse_args()

    klayout_call = ['klayout']
    klayout_call += ['-z']
    klayout_call += ['-rd', 'infile={}'.format(args.infile.name)]
    if args.ymlfile is not None:
        klayout_call += ['-rd', 'ymlfile={}'.format(args.ymlfile)]
    if args.outfile is not None:
        klayout_call += ['-rd', 'outfile={}'.format(args.outfile)]

    launcher = os.path.join(os.path.dirname(__file__), 'klayout_launcher.py')
    launcher = os.path.realpath(launcher)
    klayout_call += ['-r', launcher]

    print(' '.join(klayout_call), '\n')
    try:
        retstatus = subprocess.run(klayout_call, shell=False, check=True,
                                   stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
    except subprocess.CalledProcessError as err:
        print(err.output.decode())

# When the technology starts working, use this instead

# def cm_main():
#     ''' This one uses the klayout standalone '''
#     args = parser.parse_args()
#     batch_main(args.infile.name, ymlfile=args.ymlfile, outfile=args.outfile)





from __future__ import division, print_function, absolute_import
''' These functions and objects are in the SiEPIC-Tools project.

    Since we don't need very many they are rewritten to keep compatibility
'''
from lygadgets.environment import pya


_active_technology_name = 'OLMAC'
def active_technology():
    if pya is None:
        return None
    return pya.Technology.technology_by_name(_active_technology_name)


def tech_layer_properties():
    ''' Returns the file containing the main layer properties
    '''
    pya_tech = active_technology()
    return pya_tech.eff_path(pya_tech.eff_layer_properties_file())


def tech_dataprep_layer_properties():
    ''' Returns the file containing the main layer properties
    '''
    pya_tech = active_technology()
    return pya_tech.eff_path('dataprep/klayout_layers_dataprep.lyp')


def get_layout_variables_no_tech(cell=None):
    ''' Similar to get_layout_variables, except without looking at TECHNOLOGY
        and with the option to specify a cell
    '''
    lv = pya.Application.instance().main_window().current_view()
    if lv == None:
        print("No view selected")
        raise UserWarning("No view selected. Make sure you have an open layout.")
    if cell is None:
        # Find the currently selected layout.
        ly = lv.active_cellview().layout()
        if ly is None:
            raise UserWarning("No layout. Make sure you have an open layout.")
        # find the currently selected cell:
        cell = lv.active_cellview().cell
        if cell is None:
            raise UserWarning("No cell. Make sure you have an open layout.")
    else:
        ly = cell.layout()
    return lv, ly, cell



from lymask import batch_main

global infile, ymlfile, outfile
try:
    ymlfile
except NameError:
    ymlfile = None
try:
    outfile
except NameError:
    outfile = None

batch_main(infile, ymlfile=ymlfile, outfile=outfile)



from __future__ import division, print_function, absolute_import
from lygadgets import pya, message
import os
from lymask.siepic_utils import get_layout_variables_no_tech, active_technology, tech_dataprep_layer_properties
from lymask.soen_utils import lys, insert_layer_tab
from functools import wraps


### Helpers ###

try:
    dbu = active_technology().dbu
except AttributeError:
    dbu = .001


def as_region(cell, layname):
    ''' Just a convenience brevity function '''
    return pya.Region(cell.shapes(lys[layname]))


def filter_large_polygons(unfiltered_region, threshold_points=20):
    '''
        For each polygon, if it has too many points, its bbox will be returned instead.
        This is used to accelerate later sizing operations.
    '''
    filtered = pya.Region()
    # unfiltered_region.merged_semantics = False
    # unfiltered_region.smooth(.05 / dbu)
    for ss in unfiltered_region.each():
        if ss.num_points() > threshold_points:
            filtered.insert(ss.bbox())
        else:
            filtered.insert(ss)
    return filtered


all_func_dict = {}
def dpStep(step_fun):
    ''' Each step must accept one argument that is cell, plus optionals, and not return

        steps are added to all_steps *in the order they are defined*
    '''
    all_func_dict[step_fun.__name__] = step_fun
    return step_fun


### Entry points ###
import yaml
def _main(layout, ymlfile):
    with open(ymlfile) as fx:
        step_list = yaml.load(fx)
    for func_info in step_list:
        func = all_func_dict[func_info[0]]
        try:
            kwargs = func_info[1]
        except IndexError:
            kwargs = dict()
        for TOP_ind in layout.each_top_cell():
            # call it
            func(layout.cell(TOP_ind), **kwargs)
    return layout


def gui_main(ymlfile=None):
    lv, layout, _ = get_layout_variables_no_tech()
    lys.active_layout = layout

    insert_layer_tab(tech_dataprep_layer_properties(), tab_name='Dataprep')

    lv.transaction('Mask Dataprep')
    try:
        processed = _main(layout, ymlfile=ymlfile)
    finally:
        lv.commit()


def batch_main(infile, ymlfile=None, outfile=None):
    if outfile is None:
        outfile = infile[:-4] + '_proc.gds'
    # Load it
    layout = pya.Layout()
    layout.read(infile)
    lys.active_layout = layout
    # Find the yml file
    if ymlfile is None:
        # default dataprep test
        ymlfile = active_technology().eff_path('dataprep/test.yml')
    elif os.path.exists(os.path.realpath(ymlfile)):
        pass
    else:
        # find path to tech
        ymlfile = active_technology().eff_path(os.path.join('dataprep', ymlfile))
    # Process it
    lys.appendFile(tech_dataprep_layer_properties())
    processed = _main(layout, ymlfile=ymlfile)
    # Write it
    processed.write(outfile)
